@page brief_architecture Understanding the project architecture

* everything under <font color="tomato">Builtins</font> is related to builtin functions, used with the bytecode instructions `BUILTIN id`
    * adding one will need to reference it in `include/Ark/Builtins/Builtins.hpp` and in `src/arkreactor/Builtins/Builtins.cpp`, + implementing it accordingly under `src/arkreactor/Builtins/[file].cpp`
* the <font color="steelblue">Lexer</font>, <font color="slateblue">Parser</font>, AST generation (using Nodes), AST optimizer and <font color="skyblue">Compiler</font> are located under `include/Ark/Compiler/`.
    * the <font color="skyblue">Compiler</font> calls the <font color="slateblue">Parser</font> on a given piece of code
        * the <font color="slateblue">Parser</font> calls the <font color="steelblue">Lexer</font> on the given code
            * the <font color="steelblue">Lexer</font> returns a list of tokens
        * the <font color="slateblue">Parser</font> returns an AST from the token list
    * the <font color="skyblue">Compiler</font> calls the AST optimizer on the <font color="slateblue">Parser</font>'s AST
    * the <font color="skyblue">Compiler</font> generated bytecode, which can be used as is by the <font color="palevioletred">virtual machine</font>, or be saved to a file
* the <font color="yellowgreen">REPL</font> (read eval print loop) is located under `include/Ark/REPL/`. Basically it's an abstraction level over `replxx` (external library for completion and coloration in the shell) and our <font color="palevioletred">virtual machine</font> to run user inputs
* the <font color="palevioletred">virtual machine</font> lies under `include/Ark/VM/` and all the folders under it
    * it handles the Closures which capture whole Scopes through `shared_ptr`. Closures are functions with a saved scope, so they can retain information over multiple calls
    * the Plugin loader is a generic DLL / SO / DYNLIB loader, used by the <font color="palevioletred">virtual machine</font> to load ArkScript modules (`.arkm` files)
    * the Scope is how we store our mapping `variable id => value`, heavily optimized for our needs
    * the <font color="plum">State</font> is:
        * reading bytecode
        * decoding it
        * filling multiple tables with it (symbol table, value table, code pages), which are then used by the <font color="palevioletred">virtual machine</font>. It allows us to load a single ArkScript bytecode file and use it in multiple <font color="palevioletred">virtual machines</font>.
        * the <font color="plum">State</font> retains tables which are **never altered** by the <font color="palevioletred">virtual machines</font>
        * it can also compile ArkScript code and files on the go, and run them right away
    * the UserType is how we store C++ types unknown to our <font color="palevioletred">virtual machine</font>, to use them in ArkScript code
    * the Value is a very big proxy class to a `variant` to store our types (our custom String, double, Closure, UserType and more), thus **it must stay small** because it's the primitive type of the <font color="palevioletred">virtual machine</font> and the language
        * it handles constness and type through a tag, alongside the value
        * it provides proxy functions to the underlying `variant`
    * the <font color="palevioletred">virtual machine</font> handles:
        * the stack, a single `array<Value, 8192>` (the stack size is a define, thus it can be changed at compile time only)
        * a reference to the <font color="plum">State</font>, to read the tables and code segments
        * a `void*` user_data, retrievable by modules and C++ user functions
        * the scopes, and their destruction
        * the instructions, executed in `safeRun` which is enclosed in a try/catch to display tracebacks when errors occur
        * external function calls through its private `call` method (to call modules' functions and builtins)
        * value resolving, useful for modules when a function receives an ArkScript function, through the public method `resolve`

The `src/` folder is divided in two subfolders:
- `arkreactor/`, the <font color="skyblue">compiler</font> and the runtime
- `arkscript/`, the CLI and the <font color="yellowgreen">REPL</font>

If you feel that this section is lacking information, please open an issue [on the main repository](https://github.com/ArkScript-lang/Ark).
